@@ -15,6 +15,7 @@ from typing import Any
 
 import aiohttp
 from hydroqc.error import HydroQcError, HydroQcHTTPError
+from hydroqc.hydro_api.cache import CCached
 from hydroqc.hydro_api.consts import (
     ANNUAL_DATA_URL,
     AUTH_URL,
@@ -39,8 +40,8 @@ from hydroqc.winter_credit.consts import EST_TIMEZONE
 from hydroqc.types import (
     ComptesTyping,
     ConsoAnnualTyping,
-    ConsoDailyResultTyping,
-    ConsoHourlyResultTyping,
+    ConsoDailyTyping,
+    ConsoHourlyTyping,
     ConsoMonthlyTyping,
     InfoCompteTyping,
     PeriodDataTyping,
@@ -56,6 +57,7 @@ class HydroClient:
     _id_token: str
     access_token: str
     _selected_customer: str
+    _selected_contract: str
     cookies: dict[str, Any]
 
     def __init__(
@@ -91,6 +93,7 @@ class HydroClient:
         self.access_token = ""
         self.cookies = {}
         self._selected_customer = ""
+        self._selected_contract = ""
 
     async def http_request(
         self,
@@ -313,18 +316,23 @@ class HydroClient:
         """Return the current selected customer."""
         return self._selected_customer
 
-    async def _select_customer(
-        self, webuser_id: str, customer_id: str, force: bool = False
+    @property
+    def selected_contract(self) -> str:
+        """Return the current selected contract."""
+        return self._selected_contract
+
+    async def _select_contract(
+        self, webuser_id: str, customer_id: str, contract_id: str, force: bool = False
     ) -> None:
         """Select a customer on the Home page.
 
         Equivalent to click on the customer box on the Home page.
         """
-        # Do nothing if self._selected_customer is already the good one
-        if self._selected_customer == customer_id and not force:
+        # Do nothing if self._selected_contract is already the good one
+        if self._selected_contract == contract_id and not force:
             return
 
-        self._logger.info("Selecting customer %s", customer_id)
+        self._logger.info("Selecting contract %s", contract_id)
         if force and "cl-ec-spring.hydroquebec.com" in self.cookies:
             del self.cookies["cl-ec-spring.hydroquebec.com"]
 
@@ -337,11 +345,14 @@ class HydroClient:
         # load overview page
         # await self.http_request(CONTRACT_URL_3, "get", headers=headers)
         # load consumption profile page
-        await self.http_request(PORTRAIT_URL, "get", headers=headers)
+        params = {"noContrat": contract_id}
+        await self.http_request(PORTRAIT_URL, "get", params=params, headers=headers)
 
+        self._selected_contract = contract_id
         self._selected_customer = customer_id
-        self._logger.info("Customer %s selected", customer_id)
+        self._logger.info("Contract %s selected", contract_id)
 
+    @CCached(ttl=86400)
     async def get_user_info(self) -> list[ComptesTyping]:
         """Fetch user ids and customer ids.
 
@@ -358,6 +369,7 @@ class HydroClient:
         data: list[ComptesTyping] = await res.json()
         return data
 
+    @CCached(ttl=86400)
     async def get_customer_info(
         self, webuser_id: str, customer_id: str
     ) -> InfoCompteTyping:
@@ -374,11 +386,12 @@ class HydroClient:
         data: InfoCompteTyping = await api_call_response.json()
         return data
 
+    @CCached(ttl=86400)
     async def get_periods_info(
         self, webuser_id: str, customer_id: str, contract_id: str
     ) -> list[PeriodDataTyping]:
         """Fetch all periods info."""
-        await self._select_customer(webuser_id, customer_id)
+        await self._select_contract(webuser_id, customer_id, contract_id)
         headers = self._get_customer_http_headers(webuser_id, customer_id)
         res = await self.http_request(PERIOD_DATA_URL, "get", headers=headers)
         data = json.loads(await res.text())
@@ -390,6 +403,7 @@ class HydroClient:
             raise HydroQcError(f"No period found for contract {contract_id}")
         return periods
 
+    @CCached(ttl=86400)
     async def get_account_info(
         self, webuser_id: str, customer_id: str, account_id: str
     ) -> listeComptesContratsTyping:
@@ -404,6 +418,7 @@ class HydroClient:
             raise HydroQcError("Account not found")
         return accounts[account_id]
 
+    @CCached(ttl=86400)
     async def get_contract_info(
         self, webuser_id: str, customer_id: str, contract_id: str
     ) -> listeContratModelTyping:
@@ -420,6 +435,7 @@ class HydroClient:
             raise HydroQcError("Contract not found")
         return contracts[contract_id]
 
+    @CCached(ttl=3600)
     async def get_winter_credit(
         self, webuser_id: str, customer_id: str, contract_id: str
     ) -> WinterCreditDataTyping:
@@ -443,7 +459,7 @@ class HydroClient:
 
     async def get_today_hourly_consumption(
         self, webuser_id: str, customer_id: str, contract_id: str
-    ) -> ConsoHourlyResultTyping:
+    ) -> ConsoHourlyTyping:
         """Return latest consumption info (about 2h delay it seems).
 
         :return: raw JSON from hydro QC API for current day (not officially supported, data delayed)
@@ -456,18 +472,19 @@ class HydroClient:
         await self.get_hourly_consumption(
             webuser_id, customer_id, contract_id, yesterday_str
         )
-        res: ConsoHourlyResultTyping = await self.get_hourly_consumption(
+        res: ConsoHourlyTyping = await self.get_hourly_consumption(
             webuser_id, customer_id, contract_id, today_str
         )
         return res
 
+    @CCached(ttl=3600)
     async def get_hourly_consumption(
         self, webuser_id: str, customer_id: str, contract_id: str, date_wanted: str
-    ) -> ConsoHourlyResultTyping:
+    ) -> ConsoHourlyTyping:
         """Return hourly consumption for a specific day.
 
         .. todo::
-            Use decorator for self._select_customer
+            Use decorator for self._select_contract
 
         :param: date: YYYY-MM-DD string to pass to API
 
@@ -477,15 +494,16 @@ class HydroClient:
             "Fetching hourly consumption: c-%s c-%s", customer_id, contract_id
         )
         # TODO use decorator
-        await self._select_customer(webuser_id, customer_id)
+        await self._select_contract(webuser_id, customer_id, contract_id)
         params = {"date": date_wanted}
         api_call_response = await self.http_request(
             HOURLY_CONSUMPTION_API_URL, "get", params=params
         )
         # We can not use res.json() because the response header are not application/json
-        data: ConsoHourlyResultTyping = json.loads(await api_call_response.text())
+        data: ConsoHourlyTyping = json.loads(await api_call_response.text())
         return data
 
+    @CCached(ttl=43200)
     async def get_daily_consumption(
         self,
         webuser_id: str,
@@ -493,11 +511,11 @@ class HydroClient:
         contract_id: str,
         start_date: str,
         end_date: str,
-    ) -> ConsoDailyResultTyping:
-        """Return hourly consumption for a specific day.
+    ) -> ConsoDailyTyping:
+        """Return daily consumption for a specific day.
 
         .. todo::
-            Use decorator for self._select_customer
+            Use decorator for self._select_contract
 
         :param: start_date: YYYY-MM-DD string to pass to API
         :param: end_date: YYYY-MM-DD string to pass to API
@@ -508,7 +526,7 @@ class HydroClient:
             "Fetching daily consumption: c-%s c-%s", customer_id, contract_id
         )
         # TODO use decorator
-        await self._select_customer(webuser_id, customer_id)
+        await self._select_contract(webuser_id, customer_id, contract_id)
         params = {"dateDebut": start_date, "dateFin": end_date}
         api_call_response = await self.http_request(
             DAILY_CONSUMPTION_API_URL,
@@ -516,16 +534,17 @@ class HydroClient:
             params=params,
         )
         # We can not use res.json() because the response header are not application/json
-        data: ConsoDailyResultTyping = json.loads(await api_call_response.text())
+        data: ConsoDailyTyping = json.loads(await api_call_response.text())
         return data
 
+    @CCached(ttl=86400)
     async def get_monthly_consumption(
         self, webuser_id: str, customer_id: str, contract_id: str
     ) -> ConsoMonthlyTyping:
         """Fetch data of the current year.
 
         .. todo::
-            Use decorator for self._select_customer
+            Use decorator for self._select_contract
 
         API URL: https://cl-ec-spring.hydroquebec.com/portail/fr/group/clientele/
         portrait-de-consommation/resourceObtenirDonneesConsommationMensuelles
@@ -534,7 +553,7 @@ class HydroClient:
             "Fetching monthly consumption: c-%s c-%s", customer_id, contract_id
         )
         # TODO use decorator
-        await self._select_customer(webuser_id, customer_id)
+        await self._select_contract(webuser_id, customer_id, contract_id)
         headers = {"Content-Type": "application/json"}
         api_call_response = await self.http_request(
             MONTHLY_DATA_URL, "get", headers=headers
@@ -543,6 +562,7 @@ class HydroClient:
         data: ConsoMonthlyTyping = json.loads(await api_call_response.text())
         return data
 
+    @CCached(ttl=86400)
     async def get_annual_consumption(
         self, webuser_id: str, customer_id: str, contract_id: str
     ) -> ConsoAnnualTyping:
@@ -554,7 +574,7 @@ class HydroClient:
         self._logger.info(
             "Fetching annual consumption: c-%s c-%s", customer_id, contract_id
         )
-        await self._select_customer(webuser_id, customer_id)
+        await self._select_contract(webuser_id, customer_id, contract_id)
         headers = {"Content-Type": "application/json"}
         api_call_response = await self.http_request(
             ANNUAL_DATA_URL, "get", headers=headers
